'''

Christian Pluchar - SURF17
Data Analysis

Tool for Analyzing the large data sets (20 million+ rows) created 
in the photon absorbtion analysis.
Combines multiple CSV datasets in the same folder, finds the mean and 
stdev of each point, then produces the resulting plots specified by
makePlots.py

'''

import numpy as np
import pandas as pd
import math
import glob
import os
import csv

import matplotlib

matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
from scipy.optimize import curve_fit

from makePlots import *

directory = 'data_7_24//LED//data//all_data//group1//'

os.chdir(directory)

sampleSizes = [15,15, 15, 15, 15, 15]

#tables.shape[0] = number of rows
#tables.shape[1] = number of columns
#tables.shape[2] = depth of table (number of datasets)

#tables: 3D array
#column 0 contains mean of the datapoint, column 1 contains stdevs
#ie tables[3, 1, 2] returns the standard deviation of the 4th datapoint in the 3rd dataset
#sample sizes: list of the sample sizes of each of the datasets you want to combine
#trueMeans: array of the means for 
#takes 
def calculateStDev(table, sampleSizes, trueMeans):
	standardDeviations = np.zeros_like(trueMeans)

	for row in range(0, len(trueMeans)):
		numerator = 0
		denominator = 0
		for index in range(0, table.shape[2]):
			numerator = numerator + (sampleSizes[index]-1) * (table[row, 1, index]**2) + sampleSizes[index] * ((table[row, 0, index]**2) - (trueMeans[row]**2))

			denominator = sampleSizes[index] + denominator


		denominator -= 1

		
		stdEntry = math.sqrt(numerator / denominator)
		standardDeviations[row] = stdEntry

	return standardDeviations

#finds means for each of the rows 
#takes into account the sample size of each of the datasets
#table is 3D array (see definition above)
def calculateMeans(table, sampleSizes):
	means = np.zeros(table.shape[0])
	for row in range(0, len(means)):
		numerator = 0
		denominator = 0
		for index in range(0, table.shape[2]):
			numerator = numerator + (table[row, 0, index] * sampleSizes[index])
			denominator = denominator + sampleSizes[index]
		means[row] = numerator / denominator


	return means 

# Read the files
dataSetNames = []

for dataSetName in glob.glob('*.csv'):
	dataSetNames.append(dataSetName)

# os.chdir('..')
# os.chdir('..')
# os.chdir('..')

print(os.getcwd())
#creates 3D np array with height equal to the imported data set height,
#
dataList = []
for dataSet in dataSetNames:
	print(dataSet)
	dataList.append(pd.read_csv(dataSet, low_memory=False, header=None))

	#print("Total rows " + dataSet + ": {0}".format(len()))

# data1 = pd.read_csv("temp2data.csv", low_memory=False, header=None)
# # Output the number of rows
# print("Total rows data1: {0}".format(len(data1)))

# data2 = pd.read_csv("temp3data.csv", low_memory=False, header=None)
# print("Total rows data2: {0}".format(len(data2)))

all_data = np.dstack(dataList)

trueMeans = calculateMeans(all_data, sampleSizes)

stDev = calculateStDev(all_data, sampleSizes, trueMeans)

popt, pcov = makePlots(trueMeans, stDev)


