import matplotlib
import numpy as np
matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
import matplotlib.patches as mpatches 
from mpl_toolkits.mplot3d import Axes3D
from scipy.optimize import curve_fit

power_Stats = np.array([
[9.70955847e-10, 2.18925576e-09, 9.02221863e-10],
[1.71899093e-10, 0.00000000e+00, 1.63781079e-10],
[1.36842526e-10, 0.00000000e+00, 1.37909495e-10],
[5.26867081e-11, 0.00000000e+00, 4.97572121e-11],
[3.84819876e-12, 0.00000000e+00, 3.89454545e-12]])


transmitivity_factor = .338
power_Stats = np.sum(power_Stats, axis=1)
print(np.sum(power_Stats) / transmitivity_factor)

dark_Offset_Stats = np.array([  4.92955463e-11, 1.86589863e-14, 1.67225805e-14, 4.99848701e-15
, 7.31485905e-16])


saturated_pixels = np.array([3970, 909, 86, 4, 0])

exposure_times = np.array([30, 10, 3, 1, 1/3])






for index in range(0, power_Stats.size):
	power_Stats[index] = power_Stats[index] / transmitivity_factor
	dark_Offset_Stats[index] = dark_Offset_Stats[index] / transmitivity_factor



def integrate(npArray):
	integrated = np.zeros(npArray.size)
	for index in range(0, npArray.size):
		if index == 0:
			integrated[0] = npArray[0]
		else:
			integrated[index] = npArray[index] + integrated[index - 1]

	return integrated

powerIntegrated = integrate(power_Stats)
darkOffsetIntegrated = integrate(dark_Offset_Stats)

print(powerIntegrated)
print(darkOffsetIntegrated)
plot1 = plt.figure(1)

plt.scatter(exposure_times, powerIntegrated, color = 'red')
plt.scatter(exposure_times, darkOffsetIntegrated, color = 'black')
plt.title('Extend Dynamic Range - Power Contributions (ITMx)')
plt.xlabel('Exposure Time (S)')
plt.ylabel('Scattered Power (W)')
plt.ylim(0, 2e-08)
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))

red_patch = mpatches.Patch(color='red', label='Scattered Power')
black_patch = mpatches.Patch(color='black', label='Dark Offset Power')

plt.legend(handles=[red_patch, black_patch])



plot2 = plt.figure(2)
plt.scatter(exposure_times, saturated_pixels)

plt.title('Saturated Pixels vs Exposure Time (ITMx)')
plt.xlabel('Exposure Time (S)')
plt.ylabel('Saturated Pixels')


plot3 = plt.figure(3)

plt.scatter(exposure_times, darkOffsetIntegrated)
plt.title('Extend Dyanmic Range - DO Correction Contribution (ITMx)')
plt.xlabel('Exposure Time (S)')
plt.ylabel('Dark Offset Subtraction (W)')
plt.ylim(0, 2e-10)

plt.show()



