'''
Extend Dynamic Range

Christian Pluchar- SURF17


Starts with slowest shutter speed of an image set, and finds all the pixels which are not saturated. Adds them up, and calculate the power coming from those pixels.
Then, looks at the next slowest shutter speed, finds the not saturated pixels that were not measured in the previous image, and calculates the power from those.
Repeat until there are no saturated pixels left or at the last image. If at the last image, then take all of the remaining pixels and add them up.
'''


import rawpy
import imageio
import numpy as np
from PIL import Image
import glob
import os
import csv

reference_image_path = ''
num_channels = 3


directory = 'ITMx//previousPictures//'
saturated_value = 2**14 - 1
num_channels = 3

#Enter previously found calibration values
#red, green, blue
#in units of joules/count

calibration_values = np.array([1/(1.61e16), (1/(1.45e16)), (1/(1.65E+16))])

#camera calibration doe at ISO 200, linear scaling factors for different ISOs
#ie 1 for ISO 200, 4 for ISO 800
iso_factors = np.array(
	[100/200, 100/200, 100/200, 100/200, 100/200])

#in seconds
#slowest to fastest
exposure_times = np.array([30, 10, 3, 1, 1/3])

ambient_background_subtraction = None
dark_offset_subtraction = None

#rgb
#for ISO100, measured in the lab
#reversed currently
dark_offset_values = np.array([
[2759141.33333, 4861206.66667, 2667932.66667],
[3432974.0, 3724292.66667, 3345004.33333], 
[3941969.0, 3064002.33333, 3862679.66667],
[1703936.75, 5943926.5, 3647946.0],
[5987781.0, 6650386.0, 6249196.0]
])

#power output found by the y-intercept of the ambient noise graph
background_subtraction_values = []

#transmitivity of the lens
lens_transmitivity = .339

#names of folder paths to analyze
#enter in same order as the exposure times
folders = [
'30s',
'10s',
'3s',
'1s',
'Third'
]

#keeps track of where to add the next image in the 2d array of allImages
counter = 0 

#stores where all the images are
allImages = []

#gets all the images organized in the folders and adds them to allImages
#NOTE: will have to change the directory back the location of measureEnergy.py
for folder in folders:
	os.chdir(directory+folder)
	allImages.append([])
	for image in glob.glob('*.NEF'):
		allImages[counter].append(image)
		print(image)
	os.chdir('..')
	os.chdir('..')
	os.chdir('..')
	counter = counter + 1


#main method for analyzing the images
#returns a list with: redMean, greenMean, blueMean, saturated pixels
#for a folder of image names
def getPixelCounts(folder, folderName, saturatedPixels):

	print(folderName, 'contains ', len(folder), 'images in it')

	#keeps track of all of the pixel values extracted from the image
	#Ends up being a 2d list
	#0 -red
	#1 -green
	#2 -blue
	npPixelCounts = np.zeros([len(folder), num_channels])


	
	imageCounter = 0
	for path in folder:
		saturated = None
		imagePath = directory + folderName + '//' + path
		readImage = rawpy.imread(imagePath)

		print('Reading ', imagePath)

		#keeps the total for the raw pixel value extracted from each of the sites

		pixel_sum_blue = 0
		pixel_sum_green = 0
		pixel_sum_red = 0


		#!!!!!!taken from cj's code

		# Colour filter array
		cfa = readImage.raw_colors
		cfa_shape = cfa.shape
		all_colors = readImage.raw_image

		# Create boolean masks for each colour
		red_mask = np.reshape(np.in1d(cfa, 0), cfa_shape)
		green_mask = np.reshape(np.in1d(cfa, (1, 3)), cfa_shape)
		blue_mask = np.reshape(np.in1d(cfa, 2), cfa_shape)


		# Multiplying original array by masks sets all pixels 
		#   of colours other than the desired colour to 0
		red_only = np.multiply(all_colors, red_mask)
		green_only = np.multiply(all_colors,  green_mask)
		blue_only = np.multiply(all_colors, blue_mask)

		#end cj's code


		for height in range(0, readImage.sizes[0]):
			for width in range(0, readImage.sizes[1]):
				
				#if the pixel was previously found to be saturated/ didn't get counted
				if saturatedPixels[height, width] == 1:

					if red_only[height, width] == saturated_value or green_only[height,width] == saturated_value or blue_only[height, width] == saturated_value:
						saturated = True
						
					
					else: 
						saturatedPixels[height, width] = 0
						pixel_sum_red += red_only[height,width]
						pixel_sum_green += green_only[height,width]
						pixel_sum_blue += blue_only[height,width]


		if saturated:
			print(imagePath, ' has overexposed/saturated pixels')

		print('red', pixel_sum_red)
		print('green', pixel_sum_green)
		print('blue', pixel_sum_blue)
		npPixelCounts[imageCounter, 0] = pixel_sum_red
		npPixelCounts[imageCounter, 1] = pixel_sum_green
		npPixelCounts[imageCounter, 2] = pixel_sum_blue

		

		#resets it for the next image

		pixel_sum_blue = 0
		pixel_sum_green = 0
		pixel_sum_red = 0
		imageCounter += 1

		print('number of saturated pixels', np.sum(saturatedPixels))

	#find the mean and stDev for a group of images across all channels

	redMean = np.mean(npPixelCounts[:, 0], axis=0)

	greenMean = np.mean(npPixelCounts[:, 1], axis=0)

	blueMean = np.mean(npPixelCounts[:, 2], axis=0)

	print('means: ', redMean, greenMean, blueMean)

	return redMean, greenMean, blueMean, saturatedPixels

def analyzeImages(allImages, folders):
	

	imagePath = directory + folders[0] + '//' + str(allImages[0][0])
	sampleImage = rawpy.imread(imagePath)
	height = sampleImage.sizes[0]
	width = sampleImage.sizes[1]

	#keeps track of where the saturated pixels are
	#ones are  saturated, zeroes are unsaturated
	#starts out as all saturated so the program looks at all of the pixels on the first image
	saturatedPixels = np.ones((height, width))

	folderNumber = 0
	pixelCounts = np.zeros((len(folders), num_channels))
	for index in range(0, len(folders))
		sortedExposureTimes = np.sort(exposure_times)
		redMean, greenMean, blueMean, saturatedPixels = getPixelCounts(folder, folders[folderNumber], saturatedPixels)


		pixelCounts[folderNumber, 0] = redMean
		pixelCounts[folderNumber, 1] = greenMean
		pixelCounts[folderNumber, 2] = blueMean
		
		folderNumber += 1

	dataTable = np.zeros_like(pixelCounts)

	for index in range(0, pixelCounts.shape[0]):
		#energy is an array of length num_channels with pixel counts in it

		energy = np.multiply(pixelCounts[index], calibration_values)
		
		isoFactor = np.full_like(pixelCounts[index], iso_factors[index])
		energy = np.divide(energy, isoFactor)

		transmitivityFactor = np.full_like(pixelCounts[index], lens_transmitivity)
		energy = np.divide(energy, transmitivityFactor)

		print(folders[index])
		print('energy', energy)

		exposureTime = np.full_like(pixelCounts[index], exposure_times[index])
		power = np.divide(energy, exposureTime)
		
		dataTable[index] = power

	scatteredPower = np.sum(dataTable)

	if ambient_background_subtraction:
		scatteredPower -= - background_subtraction_values

	return scatteredPower

data = analyzeImages(allImages, folders)

print('power', data)





