'''

MeasurePower
Christian Pluchar SURF17

Takes a group of images, calculates the power of the ambient light
Used for measuring the scattering off the test masses
Need camera calibration before using this
Images are divided into folders ()
'''


import rawpy
import imageio
import numpy as np
from PIL import Image
import glob
import os
import csv


directory = 'ETMx//Locked//'
#14 bits
saturated_value = 2**14 - 1
#red, green, blue
num_channels = 3

#Enter previously found calibration values
#red, green, blue
#in units of joules/count

calibration_values = np.array([1/(1.61e16), (1/(1.45e16)), (1/(1.65E+16))])

print(calibration_values)

#camera calibration doe at ISO 200, linear scaling factors for different ISOs
#ie 1 for ISO 200, 4 for ISO 800
iso_factors = np.array(
	[100/200, 100/200, 100/200, 100/200, 100/200, 
	100/200, 100/200, 100/200, 100/200, 100/200, 
	100/200, 100/200, 100/200, 100/200, 100/200, 
	100/200, 100/200, 100/200, 100/200])

#in seconds
exposure_times = np.array([
	1/25, 1/30,
	1/40, 1/60, 1/125, 1/250, 1/500, 1/1000, 1/2000, 1/3200, 
	1/5000, 1/6400, 
	1/8000,
	1/15, 1/5, 1/2, 1, 1/10, 2/5
	])

ambient_background_subtraction = None
dark_offset_subtraction = None

#rgb
#formula from dark offset measurements with D7100
#See August 10th notes
def darkOffsetFunc(exposureTime):
	return  10**(-1 * np.log10(exposureTime) - 8.83)

print(darkOffsetFunc(1/8000))

#transmitivity of the lens
lens_transmitivity = 1

#names of folder paths to analyze
folders = [
'25',
'30',
'40',
'60',
'125',
'250',
'500',
'1000',
'2000',
'3200',
'5000',
'6400',
'8000',
'Fifteenth',
'Fifth',
'Half',
'One',
'Tenth',
'TwoFifths'
]



#keeps track of where to add the next image in the 2d array of allImages
counter = 0 

#stores where all the images are
allImages = []

#gets all the images organized in the folders and adds them to allImages
#NOTE: will have to change the directory back the location of measureEnergy.py
for folder in folders:
	os.chdir(directory+folder)
	allImages.append([])
	for image in glob.glob('*.NEF'):
		allImages[counter].append(image)
		print(image)
	os.chdir('..')
	os.chdir('..')
	os.chdir('..')
	counter = counter + 1

#main method for analyzing the images
#returns a list with: folderName, allMean, redMean, greenMean, blueMean
#for a folder of images
def getPixelCounts(folder, folderName):

	print(folderName, 'contains ', len(folder), 'images in it')

	#keeps track of all of the pixel values extracted from the image
	#Ends up being a 2d list
	#0 -red
	#1 -green
	#2 -blue
	npPixelCounts = np.zeros([len(folder), num_channels])
	
	imageCounter = 0
	for path in folder:
		saturated = None
		imagePath = directory + folderName + '//' + path
		readImage = rawpy.imread(imagePath)

		print('Reading ', imagePath)

		#keeps the total for the raw pixel value extracted from each of the sites

		pixel_sum_blue = 0
		pixel_sum_green = 0
		pixel_sum_red = 0


		#!!!!!!taken from cj's code

		# Colour filter array
		cfa = readImage.raw_colors
		cfa_shape = cfa.shape
		all_colors = readImage.raw_image

		# Create boolean masks for each colour
		red_mask = np.reshape(np.in1d(cfa, 0), cfa_shape)
		green_mask = np.reshape(np.in1d(cfa, (1, 3)), cfa_shape)
		blue_mask = np.reshape(np.in1d(cfa, 2), cfa_shape)


		# Multiplying original array by masks sets all pixels 
		#   of colours other than the desired colour to 0
		red_only = np.multiply(all_colors, red_mask)
		green_only = np.multiply(all_colors,  green_mask)
		blue_only = np.multiply(all_colors, blue_mask)

		#end cj's code

		for height in range(0, readImage.sizes[0]):
			for width in range(0, readImage.sizes[1]):
				if readImage.raw_value_visible(height,width) == saturated_value:
					saturated = True

				#analysis - for total pixel sum at each site, use option 1, for 
				#getting each color, use option 2
				#1:
				#pixel_sum += readImage.raw_value_visible(height,width)

				#2:
				pixel_sum_red += red_only[height,width]
				pixel_sum_green += green_only[height,width]
				pixel_sum_blue += blue_only[height,width]


		if saturated:
			print(imagePath, ' has overexposed/saturated pixels')

		print('red', pixel_sum_red)
		print('green', pixel_sum_green)
		print('blue', pixel_sum_blue)
		npPixelCounts[imageCounter, 0] = pixel_sum_red
		npPixelCounts[imageCounter, 1] = pixel_sum_green
		npPixelCounts[imageCounter, 2] = pixel_sum_blue

		

		#resets it for the next image

		pixel_sum_blue = 0
		pixel_sum_green = 0
		pixel_sum_red = 0
		imageCounter += 1

	#find the mean and stDev for a group of images across all channels

	print(npPixelCounts)

	redMean = np.mean(npPixelCounts[:, 0], axis=0)

	greenMean = np.mean(npPixelCounts[:, 1], axis=0)

	blueMean = np.mean(npPixelCounts[:, 2], axis=0)

	print('means: ', redMean, greenMean, blueMean)

	return redMean, greenMean, blueMean

def analyzeImages(allImages, folders):
	folderNumber = 0
	pixelCounts = np.zeros([len(folders), num_channels])
	for folder in allImages:
		redMean, greenMean, blueMean = getPixelCounts(folder, folders[folderNumber])
		# dataTableToSave.append([])
		# dataTableToSave[folderNumber].append([folders[folderNumber], redMean, greenMean, blueMean])


		pixelCounts[folderNumber, 0] = redMean
		pixelCounts[folderNumber, 1] = greenMean
		pixelCounts[folderNumber, 2] = blueMean
		folderNumber += 1

	dataTable = np.zeros_like(pixelCounts)

	#calculations
	for index in range(0, pixelCounts.shape[0]):
		#energy is an array of length num_channels with pixel counts in it

		#energy = (pixel counts) * (energy per pixel count)
		energy = np.multiply(pixelCounts[index], calibration_values)
		
		#iso scales linearly, this adjusts for that
		#calibration was taken with ISO200
		isoFactor = np.full_like(pixelCounts[index], iso_factors[index])
		energy = np.divide(energy, isoFactor)

		# Power = Energy / Time
		exposureTime = np.full_like(pixelCounts[index], exposure_times[index])
		power = np.divide(energy, exposureTime)
		
		dataTable[index] = power


	return dataTable

data = analyzeImages(allImages, folders)

data = np.sum(data, axis=1)

oldData = data
if dark_offset_subtraction:

	for index in range(0, data.size):
		print('old', data[index])
		data[index] -= darkOffsetFunc(exposure_times[index])
		print('new', data[index])
print('power', data)






