

import matplotlib
import numpy as np
matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from scipy.optimize import curve_fit


#in seconds
exposure_times = np.array([1/3, 1, 3, 10, 30])

powers = [2.41080615e-09, 3.77029022e-09, 4.38037350e-09, 4.35656620e-09, 4.24788279e-09]

transmitivity_factor = .339

for index in range(0, len(powers)):
	powers[index] = powers[index] / transmitivity_factor

def line(x, slope, yIntersect):
	return (x * slope + yIntersect)

plot1 = plt.figure(1)

plt.scatter(exposure_times, powers)
plt.title('Scattered Power vs Camera Exposure Time (ITMx)')
plt.xlabel('Exposure Time (S)')
plt.ylabel('Scattered Power (W)')
plt.ylim(0, 2e-8)

popt, pcov = curve_fit(line, exposure_times, powers)

xfine = np.linspace(0, np.amax(exposure_times), num = 100)
plt.plot(xfine, line(xfine, *popt), 'r-')

perr = np.sqrt(np.diag(pcov))

print(popt)
print(perr)

plt.show()